@@ -42,18 +42,20 @@ class SmarterSender:
         trial_limit = self.__smarter_store.get_manifest_property(property_sequence="price.paygTrialUnits")
         trial_limit = eval(trial_limit) if SmarterSender.__is_number(trial_limit) else None
         is_subscribed = self.__core.get("isSubscribed")
-        if not is_subscribed and trial_limit and usage_data[exp_slug] > trial_limit:
+        if not is_subscribed and trial_limit and usage_data.get(exp_slug) > trial_limit:
             # Reached the trial limit
             return -1
-        return usage_data[exp_slug]
+        return usage_data.get(exp_slug)
 
-    def __increment_usage(self, inc: int) -> Any:
+    def __increment_usage(self, inc: int, user_id: str) -> Any:
         exp_slug = self.__core.get("expSlug")
         usage_data = self.__smarter_store.read_global_store(pattern=self.__usage_key)
+        if user_id not in usage_data:
+            usage_data[user_id] = {}
         project_slug = self.__core.get("slug")
         if exp_slug not in usage_data:
-            usage_data[exp_slug] = 0
-        usage_data[exp_slug] += inc
+            usage_data[user_id][exp_slug] = 0
+        usage_data[user_id][exp_slug] += inc
         limit_reached = self.__check_limit(usage_data=usage_data, exp_slug=exp_slug)
         if limit_reached in [0, -1]:
             return limit_reached
@@ -67,10 +69,10 @@ class SmarterSender:
         self.send_message(message=new_message, port='#gui')
         return usage_data[exp_slug]
 
-    def __get_usage(self) -> int:
+    def __get_usage(self, user_id: str) -> int:
         usage_data = self.__smarter_store.read_global_store(pattern=self.__usage_key)
         exp_slug = self.__core.get("expSlug")
-        return usage_data.get(exp_slug, 0)
+        return usage_data.get(user_id, {}).get(exp_slug, 0)
 
     def send_message(self, message: SmarterMessage, port: str) -> Optional[SmarterMessage]:
         """
@@ -89,10 +91,11 @@ class SmarterSender:
                         Example a chart expects a table-like format, while a textField expects some text.
         :return: None
         """
-        self.__smarter_store.set_pattern_data(pattern=pattern, data=data)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": data}})
-        self.send_message(message=message, port='#gui')
+        return_message = self.send_message(message=message, port='#gui')
+        user_id = return_message.get("user", {}).get("id")
+        self.__smarter_store.set_pattern_data(pattern=pattern, data=data, user_id=user_id)
 
     def clear_data(self, pattern: str) -> None:
         """
@@ -100,10 +103,11 @@ class SmarterSender:
         :param pattern: The front-end GUI pattern to set the data to
         :return: None
         """
-        self.__smarter_store.set_pattern_data(pattern=pattern, data=None)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": None}})
-        self.send_message(message=message, port='#gui')
+        return_message = self.send_message(message=message, port='#gui')
+        user_id = return_message.get("user", {}).get("id")
+        self.__smarter_store.set_pattern_data(pattern=pattern, data=None, user_id=user_id)
 
     def append_data(self, pattern: str, data: Any, limit: int) -> None:
         """
@@ -114,11 +118,12 @@ class SmarterSender:
             the latest added elements
         :return: None
         """
-        self.__smarter_store.append_pattern_data(pattern=pattern, data=data)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": data if type(data) == list else [data]},
                                   "options": {"append": True, "limitLength": limit}})
-        self.send_message(message=message, port='#gui')
+        return_message = self.send_message(message=message, port='#gui')
+        user_id = return_message.get("user", {}).get("id")
+        self.__smarter_store.append_pattern_data(pattern=pattern, data=data, user_id=user_id)
 
     def prepend_data(self, pattern: str, data: Any, limit: int) -> None:
         """
@@ -129,11 +134,12 @@ class SmarterSender:
             the latest added elements
         :return: None
         """
-        self.__smarter_store.prepend_pattern_data(pattern=pattern, data=data)
         message = SmarterMessage({"action": "setData",
                                   "args": {"pattern": pattern, "data": data if type(data) == list else [data]},
                                   "options": {"prepend": True, "limitLength": limit}})
-        self.send_message(message=message, port='#gui')
+        return_message = self.send_message(message=message, port='#gui')
+        user_id = return_message.get("user", {}).get("id")
+        self.__smarter_store.prepend_pattern_data(pattern=pattern, data=data, user_id=user_id)
 
     def get_data(self, pattern: str) -> Any:
         """
@@ -141,7 +147,9 @@ class SmarterSender:
         :param pattern: The pattern to return
         :return: json serializable SmarterMessage
         """
-        return self.__smarter_store.get_pattern_data(pattern)
+        return_message = self.send_message(message=SmarterMessage(), port="#user")
+        user_id = return_message.get("user", {}).get("id")
+        return self.__smarter_store.get_pattern_data(pattern=pattern, user_id=user_id)
 
     def reply_back(self, message: SmarterMessage) -> None:
         """
@@ -225,7 +233,9 @@ class SmarterSender:
                  Returns 0 if no trial limit was set
                  Otherwise returns a value >= 1 depending on the user's current usage after increment.
         """
-        return self.__increment_usage(increment_value)
+        return_message = self.send_message(message=SmarterMessage(), port="#user")
+        user_id = return_message.get("user", {}).get("id")
+        return self.__increment_usage(increment_value, user_id=user_id)
 
     def get_usage(self) -> int:
         """
@@ -234,7 +244,9 @@ class SmarterSender:
                  Returns 0 if no trial limit was set or if the user is the solution's creator
                  Otherwise returns a value >= 1 depending on the user's current usage.
         """
-        return self.__get_usage()
+        return_message = self.send_message(message=SmarterMessage(), port="#user")
+        user_id = return_message.get("user", {}).get("id")
+        return self.__get_usage(user_id=user_id)
 
     def get_event_collection(self):
         return self.__db.get_collection(collection_name=f"events_{self.__user_id}")
@@ -247,8 +259,8 @@ class SmarterSender:
         result = collection.find({"userEmail": self.__user_email})
         return list(result)
 
-    def add_tag(self, is_event, id, new_tag, new_val):
-        self.__db.add_tag(is_event=is_event, id=id, new_tag=new_tag, new_val=new_val, user_id=self.__user_id)
+    def add_tag(self, is_event, tag_id, new_tag, new_val):
+        self.__db.add_tag(is_event=is_event, tag_id=tag_id, new_tag=new_tag, new_val=new_val, user_id=self.__user_id)
 
 
 class _Smarter_SignatureCheckerMeta(ABCMeta):
